const asyncHandler = require("express-async-handler");
const UserModel = require("../models/UserModel");
const { send, Validate } = require("../utils/logic/otp");
const { extractPhoneDetails, generateToken } = require("../utils/logic/helper");
const ApiError = require("../utils/apiError");
const jwt = require("jsonwebtoken");

exports.login = asyncHandler(async (req, res, next) => {
  const { phone } = req.body;

  const existingUser = await UserModel.findOne({ phone });

  if (!existingUser) {
    return res.status(404).json({
      success: false,
      message: "User not found ",
    });
  }

  if (existingUser.status === false) {
    return res.status(403).json({
      success: false,
      message:
        "Your account is already deactivated. Please contact support for help ",
    });
  }
  let countryCode, mobileNumber;
  try {
    ({ countryCode, mobileNumber } = extractPhoneDetails(phone));
  } catch (error) {
    console.error(error.message);
  }

  const test = await send(countryCode, mobileNumber);
  res.status(200).json({ success: true, data: test });
});

exports.register = asyncHandler(async (req, res, next) => {
  const { name, phone } = req.body;
  let countryCode, mobileNumber;
  try {
    ({ countryCode, mobileNumber } = extractPhoneDetails(phone));
  } catch (error) {
    console.error(error.message);
  }

  const data = { name, phone };

  const ExistingUser = await UserModel.findOne({ phone });
  if (ExistingUser) {
    next(new ApiError(` ${name} already exists`, 400));
    return;
  }
  await UserModel.create(data);
  // eslint-disable-next-line camelcase
  const send_otp = await send(countryCode, mobileNumber);

  // eslint-disable-next-line camelcase
  res.status(200).json(send_otp);
});

exports.ValidateOtp = asyncHandler(async (req, res, next) => {
  const { verificationId, phone, code } = req.body;

  let countryCode, mobileNumber;
  try {
    ({ countryCode, mobileNumber } = extractPhoneDetails(phone));
  } catch (error) {
    return next(new ApiError("Invalid phone format", 400));
  }

  const existingUser = await UserModel.findOne({ phone });
  if (!existingUser) {
    return next(new ApiError("User does not exist", 404));
  }

  if (existingUser.role !== "google_play") {
    const validateOtpResponse = await Validate(
      verificationId,
      countryCode,
      mobileNumber,
      code
    );

    console.log("validateOtpResponse:", validateOtpResponse);
    if (
      validateOtpResponse.data.verificationStatus !== "VERIFICATION_COMPLETED"
    ) {
      return next(new ApiError("Invalid OTP code", 400));
    }
  }
  console.log("id:", existingUser._id);
  await UserModel.findOneAndUpdate(
    { phone },
    { isPhoneVerified: true, status: true, $inc: { sessionVersion: 1 } },
    { new: true }
  );
  // Generate JWT token
  const token = generateToken(
    existingUser._id,
    phone,
    existingUser.sessionVersion + 1
  );
  console.log("Generated token:", token);

  res.status(200).json({
    success: true,
    message: "OTP validated successfully",
    user: existingUser,
    token: token,
  });
});

exports.logout = asyncHandler(async (req, res, next) => {
  await UserModel.findByIdAndUpdate(req.user._id, {
    $inc: { sessionVersion: 1 },
  });
  
  res
    .status(200)
    .json({ message: "Logged out successfully from all devices." });
});

exports.auth = asyncHandler(async (req, res, next) => {
  // 1) Check if token exist, if exist get
  let token;
  if (
    req.headers.authorization &&
    req.headers.authorization.startsWith("Bearer")
  ) {
    token = req.headers.authorization.split(" ")[1];
  }
  if (!token) {
    return next(
      new ApiError(
        "You are not login, Please login to get access this route",
        401
      )
    );
  }
  // console.log('process.env.JWT_SECRET_KEY:', process.env.JWT_SECRET);
  // 2) Verify token (no change happens, expired token)
  const decoded = jwt.verify(token, process.env.JWT_SECRET);
  //console.log('Decoded:', decoded);

  // 3) Check if user exists
  const currentUser = await UserModel.findById(decoded.userID);
  if (!currentUser || currentUser.sessionVersion !== decoded.sessionVersion) {
    return next(new ApiError("Session expired. Please log in again", 401));
  }

  req.user = currentUser;
  next();
});

exports.admin = asyncHandler(async (req, res, next) => {
  // 1) Check if token exist, if exist get
  let token;
  if (
    req.headers.authorization &&
    req.headers.authorization.startsWith("Bearer")
  ) {
    token = req.headers.authorization.split(" ")[1];
  }
  if (!token) {
    return next(
      new ApiError(
        "You are not login, Please login to get access this route",
        401
      )
    );
  }
  // console.log('process.env.JWT_SECRET_KEY:', process.env.JWT_SECRET);
  // 2) Verify token (no change happens, expired token)
  const decoded = jwt.verify(token, process.env.JWT_SECRET);
  //console.log('Decoded:', decoded);

  // 3) Check if user exists
  const currentUser = await UserModel.findById(decoded.userID);
  if (!currentUser || currentUser.sessionVersion !== decoded.sessionVersion) {
    return next(
      new ApiError(
        "The user that belongs to this token does no longer exist",
        401
      )
    );
  }

  if (currentUser.role !== "admin") {
    return next(
      new ApiError("You do not have permission to access this route", 403)
    );
  }

  req.user = currentUser;
  next();
});
