const slugify = require("slugify");

const path = require("path");
const asyncHandler = require("express-async-handler");
const CategoryModel = require("../models/CategoryModel");
const UserModel = require("../models/UserModel");
const ApiError = require("../utils/apiError");
const CategoryResource = require("../utils/resources/CategoryResource");
const { imageClear } = require("../utils/logic/helper");
const { uploadSingleImage } = require("../Middlewares/uploadImageMiddleware");

exports.UploadImage = uploadSingleImage("image", "images/category");

exports.GetCategories = asyncHandler(async (req, res) => {
  try {
    const lang = req.headers["accept-language"] || "en";
    const user_id = req.user ? req.user._id : null;

    const page = req.query.page * 1 || 1;
    const limit = req.query.limit * 1 || 100;
    const skip = (page - 1) * limit;

    let categories = await CategoryModel.find({}).skip(skip).limit(limit);

    const data = await CategoryResource.collection(categories, lang, user_id);

    res.status(200).json({
      result: data.length,
      page,
      data,
    });
  } catch (error) {
    res.status(500).json({
      success: false,
      message: "Server Error",
      error: error.message,
    });
  }
});

exports.GetCategory = asyncHandler(async (req, res, next) => {
  const { id } = req.params;
  const category = await CategoryModel.findById(id);
  if (!category) {
    next(new ApiError(`not found ${req.originalUrl}`, 400));
  }
  res.status(200).json({ data: category });
});

exports.AddCategory = async (req, res) => {
  try {
    const { name_en, name_ar, numberOfLocations } = req.body;
    const image = req.file.path;
    // Debugging logs to ensure correct data is being received
    console.log("Request Body:", req.body);

    // Validate required fields
    if (!name_en || typeof name_en !== "string" || name_en.trim() === "") {
      return res
        .status(400)
        .json({ error: "name_en is required and cannot be empty." });
    }

    if (!name_ar || typeof name_ar !== "string" || name_ar.trim() === "") {
      return res
        .status(400)
        .json({ error: "name_ar is required and cannot be empty." });
    }

    // Check if category already exists
    const existingCategory = await CategoryModel.findOne({
      $or: [{ name_en }, { name_ar }],
    });

    if (existingCategory) {
      return res
        .status(400)
        .json({ error: "Category with this name already exists." });
    }

    // Create new category
    const newCategory = await CategoryModel.create({
      name_en,
      name_ar,
      slug: slugify(name_en),
      image,
      numberOfLocations,
    });

    res.status(200).json({
      success: true,
      category: newCategory,
    });
  } catch (error) {
    console.error("Error adding category:", error);
    res.status(500).json({ error: error.message });
  }
};

exports.UpdateCategory = asyncHandler(async (req, res, next) => {
  const updateData = {};
  const { id } = req.params;
  const { name_en, name_ar } = req.body;
  const { numberOfLocations } = req.body;
  if (req.file) {
    const image = req.file.path;
    updateData.image = image;
  }
  if (name_en) {
    updateData.name_en = name_en;
    updateData.slug = slugify(updateData.name_en);
  }
  if (name_ar) {
    updateData.name_ar = name_ar;
  }
  if (numberOfLocations) {
    updateData.numberOfLocations = numberOfLocations;
  }

  const oldCategory = await CategoryModel.findOne({ _id: id });

  const category = await CategoryModel.findOneAndUpdate(
    { _id: id },
    updateData,
    { new: true }
  );

  if (
    updateData.image !== undefined &&
    updateData.image !== oldCategory.image
  ) {
    imageClear(oldCategory.image);
  }
  if (!category) {
    next(new ApiError(`not found ${req.originalUrl}`, 400));
  }
  res.status(200).json({ data: category });
});

exports.DeleteCategory = asyncHandler(async (req, res, next) => {
  const { id } = req.params;
  const category = await CategoryModel.findById(id);

  imageClear(category.image);

  if (!category) {
    const error = new Error("Category not founded ");
    error.statusCode = 422;
    throw error;
  }
  await category.deleteOne();
  res.status(200).json({ msg: "Category deleted" });
});

exports.InteresteCategoryes = async (req, res, next) => {
  const { selectedCategories } = req.body;

  const user = await UserModel.findByIdAndUpdate(
    req.user._id,
    { selectedCategories },
    { new: true }
  ).populate("status");

  if (!user) {
    return res.status(404).json({ message: "No selected category yet" });
  }

  res.status(200).json(user);
};
