const PostModel = require("../models/PostModel");
const ApiError = require("../utils/apiError");
const PostResource = require("../utils/resources/PostResource");
const { uploadSingleImage } = require("../Middlewares/uploadImageMiddleware");
const { uploadMixOfImages } = require("../Middlewares/uploadImageMiddleware");
const { GetPostOfUser } = require("../utils/logic/helper");
const { imageClear } = require("../utils/logic/helper");
exports.UploadImage = uploadSingleImage("image", "images/post");
exports.MultiImages = uploadMixOfImages("images", "images/post");

exports.GetMyPost = async (req, res, next) => {
  try {
    const userId = req.user._id;

    const posts = await GetPostOfUser(userId);

    res.status(200).json(posts);
  } catch (error) {
    console.error("Error fetching user's posts:", error);
    res.status(500).json({ message: "Something went wrong" });
  }
};

exports.AddPost = async (req, res) => {
  const { description, price, category } = req.body;
  let images = [];
  if (req.files) {
    images = req.files ? req.files.map((file) => file.path) : [];
  }
  try {
    const add = await PostModel.create({
      description,
      price,
      images,
      category,
      user: req.user._id,
    });
    res.status(200).json({ data: add });
  } catch (error) {
    if (error.code === 11000) {
      // MongoDB duplicate key error code is 11000
      res.status(409).json({
        message: "Service with the same name already exists",
        error: error.message,
      });
    }
  }
};

exports.DeletePost = async (req, res, next) => {
  try {
    const { id } = req.params;

    const post = await PostModel.findOneAndUpdate(
      { _id: id, user: req.user._id },
      { status: false },
      { new: true }
    );

    if (!post) {
      return res.status(422).json({ message: "Post not found" });
    }

    res.status(200).json({ message: "Post deleted successfully" });
  } catch (error) {
    console.error("Error deleting post:", error);
    res.status(500).json({ message: "Something went wrong" });
  }
};

exports.Deleteimage = async (req, res, next) => {
  try {
    const { id } = req.params;
    const { image } = req.body;

    const status = await imageClear(image);

    if (status) {
      const post = await PostModel.findOneAndUpdate(
        { _id: id, user: req.user._id },
        { $pull: { images: image } },
        { new: true }
      );

      if (!post) {
        return res.status(404).json({ message: "Post not found" });
      }

      res.status(200).json({ message: "Image deleted successfully", post });
    } else {
      res.status(500).json({ message: "Failed to delete the image file" });
    }
  } catch (error) {
    console.error("Error deleting image:", error);
    res.status(500).json({ message: "Something went wrong" });
  }
};

exports.Updatepost = async (req, res, next) => {
  try {
    console.log("body", req.body);
    const updateData = {};
    const { id } = req.params;
    const { description, price, category } = req.body;

    const post = await PostModel.findOne({ _id: id, user: req.user._id });
    if (!post) {
      return next(new ApiError(`Post not found at ${req.originalUrl}`, 404));
    }

    if (req.files && req.files.length > 0) {
      const newImages = req.files.map((file) => file.path);
      updateData.images = [...(post.images || []), ...newImages];
    }

    if (description !== undefined) {
      updateData.description = description;
    }
    if (price !== undefined) {
      updateData.price = price;
    }
    if (category !== undefined) {
      updateData.category = category;
    }

    const updatedPost = await PostModel.findByIdAndUpdate(id, updateData, {
      new: true,
    });

    res.status(200).json({ data: updatedPost });
  } catch (error) {
    console.error(error);
    next(error);
  }
};

exports.SearchSocket = async (
  userId,
  page = 1,
  limit = 10,
  category = "",
  search = ""
) => {
  const skip = (page - 1) * limit;

  const matchStage = {
    user: { $ne: userId }
  };

  if (category) {
    matchStage.category = category;
  }

  const searchRegex = search ? new RegExp(search, 'i') : null;

  const pipeline = [
    { $match: matchStage },
    {
      $lookup: {
        from: "users",
        localField: "user",
        foreignField: "_id",
        as: "user"
      }
    },
    { $unwind: "$user" },
    {
      $lookup: {
        from: "categories", 
        localField: "category",
        foreignField: "_id",
        as: "category"
      }
    },
    { $unwind: "$category" },
  ];

  if (searchRegex) {
    pipeline.push({
      $match: {
        $or: [
          { description: { $regex: searchRegex } },
          { "user.name": { $regex: searchRegex } }
        ]
      }
    });
  }

  pipeline.push(
    { $sort: { createdAt: -1 } },
    { $skip: skip },
    { $limit: limit }
  );
console.log(pipeline);
  const post = await PostModel.aggregate(pipeline);

  const data = await PostResource.collection(post);
  return data;
};
